package com.sun.org.apache.xml.internal.security.utils;

import com.sun.org.slf4j.internal.LoggerFactory;
import java.security.AccessController;
import java.math.BigInteger;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.DocumentBuilder;
import java.util.ArrayList;
import java.util.HashSet;
import org.w3c.dom.NodeList;
import java.util.Base64;
import java.util.Iterator;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Text;
import com.sun.org.apache.xml.internal.security.c14n.CanonicalizationException;
import com.sun.org.apache.xml.internal.security.c14n.InvalidCanonicalizerException;
import java.io.IOException;
import com.sun.org.apache.xml.internal.security.c14n.Canonicalizer;
import java.nio.charset.StandardCharsets;
import java.io.OutputStream;
import org.w3c.dom.NamedNodeMap;
import java.util.Set;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import com.sun.org.slf4j.internal.Logger;

public final class XMLUtils
{
    private static boolean ignoreLineBreaks;
    private static final boolean LINEFEED_ONLY;
    private static volatile String dsPrefix;
    private static volatile String ds11Prefix;
    private static volatile String xencPrefix;
    private static volatile String xenc11Prefix;
    private static final Logger LOG;
    private static final int MIMELINEMAX = 76;
    
    private XMLUtils() {
    }
    
    public static void setDsPrefix(String dsPrefix) {
        JavaUtils.checkRegisterPermission();
        XMLUtils.dsPrefix = dsPrefix;
    }
    
    public static void setDs11Prefix(String ds11Prefix) {
        JavaUtils.checkRegisterPermission();
        XMLUtils.ds11Prefix = ds11Prefix;
    }
    
    public static void setXencPrefix(String xencPrefix) {
        JavaUtils.checkRegisterPermission();
        XMLUtils.xencPrefix = xencPrefix;
    }
    
    public static void setXenc11Prefix(String xenc11Prefix) {
        JavaUtils.checkRegisterPermission();
        XMLUtils.xenc11Prefix = xenc11Prefix;
    }
    
    public static Element getNextElement(Node nextSibling) {
        for (; nextSibling != null && nextSibling.getNodeType() != 1; nextSibling = nextSibling.getNextSibling()) {}
        return (Element)nextSibling;
    }
    
    public static void getSet(Node node, Set<Node> set, Node node2, boolean b) {
        if (node2 != null && isDescendantOrSelf(node2, node)) {
            return;
        }
        getSetRec(node, set, node2, b);
        return;
    }
    
    private static void getSetRec(Node node, Set<Node> set, Node node2, boolean b) {
        if (node == node2) {
            return;
        }
        switch (node.getNodeType()) {
            case 1: {
                set.add(node);
                Element element = (Element)node;
                if (element.hasAttributes()) {
                    NamedNodeMap attributes = element.getAttributes();
                    for (int length = attributes.getLength(), i = 0; i < length; ++i) {
                        set.add(attributes.item(i));
                    }
                }
            }
            case 9: {
                for (Node node3 = node.getFirstChild(); node3 != null; node3 = node3.getNextSibling()) {
                    if (node3.getNodeType() == 3) {
                        set.add(node3);
                        while (node3 != null && node3.getNodeType() == 3) {
                            node3 = node3.getNextSibling();
                        }
                        if (node3 == null) {
                            return;
                        }
                    }
                    getSetRec(node3, set, node2, b);
                }
                return;
            }
            case 8: {
                if (b) {
                    set.add(node);
                }
                return;
            }
            case 10: {
                return;
            }
            default: {
                set.add(node);
                return;
            }
        }
    }
    
    public static void outputDOM(Node node, OutputStream outputStream) {
        outputDOM(node, outputStream, false);
    }
    
    public static void outputDOM(Node node, OutputStream outputStream, boolean b) {
        try {
            if (b) {
                outputStream.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n".getBytes(StandardCharsets.UTF_8));
            }
            outputStream.write(Canonicalizer.getInstance("http://santuario.apache.org/c14n/physical").canonicalizeSubtree(node));
        }
        catch (IOException e) {
            XMLUtils.LOG.debug(e.getMessage(), e);
        }
        catch (InvalidCanonicalizerException e2) {
            XMLUtils.LOG.debug(e2.getMessage(), e2);
        }
        catch (CanonicalizationException e3) {
            XMLUtils.LOG.debug(e3.getMessage(), e3);
        }
    }
    
    public static void outputDOMc14nWithComments(Node node, OutputStream outputStream) {
        try {
            outputStream.write(Canonicalizer.getInstance("http://www.w3.org/TR/2001/REC-xml-c14n-20010315#WithComments").canonicalizeSubtree(node));
        }
        catch (IOException e) {
            XMLUtils.LOG.debug(e.getMessage(), e);
        }
        catch (InvalidCanonicalizerException e2) {
            XMLUtils.LOG.debug(e2.getMessage(), e2);
        }
        catch (CanonicalizationException e3) {
            XMLUtils.LOG.debug(e3.getMessage(), e3);
        }
    }
    
    @Deprecated
    public static String getFullTextChildrenFromElement(Element element) {
        return getFullTextChildrenFromNode(element);
    }
    
    public static String getFullTextChildrenFromNode(Node node) {
        StringBuilder sb = new StringBuilder();
        for (Node node2 = node.getFirstChild(); node2 != null; node2 = node2.getNextSibling()) {
            if (node2.getNodeType() == 3) {
                sb.append(((Text)node2).getData());
            }
        }
        return sb.toString();
    }
    
    public static Element createElementInSignatureSpace(Document document, String str) {
        if (document == null) {
            throw new RuntimeException("Document is null");
        }
        if (XMLUtils.dsPrefix == null || XMLUtils.dsPrefix.length() == 0) {
            return document.createElementNS("http://www.w3.org/2000/09/xmldsig#", str);
        }
        return document.createElementNS("http://www.w3.org/2000/09/xmldsig#", XMLUtils.dsPrefix + ":" + str);
    }
    
    public static Element createElementInSignature11Space(Document document, String str) {
        if (document == null) {
            throw new RuntimeException("Document is null");
        }
        if (XMLUtils.ds11Prefix == null || XMLUtils.ds11Prefix.length() == 0) {
            return document.createElementNS("http://www.w3.org/2009/xmldsig11#", str);
        }
        return document.createElementNS("http://www.w3.org/2009/xmldsig11#", XMLUtils.ds11Prefix + ":" + str);
    }
    
    public static boolean elementIsInSignatureSpace(Element element, String anObject) {
        return element != null && "http://www.w3.org/2000/09/xmldsig#".equals(element.getNamespaceURI()) && element.getLocalName().equals(anObject);
    }
    
    public static boolean elementIsInSignature11Space(Element element, String anObject) {
        return element != null && "http://www.w3.org/2009/xmldsig11#".equals(element.getNamespaceURI()) && element.getLocalName().equals(anObject);
    }
    
    public static Document getOwnerDocument(Node node) {
        if (node.getNodeType() == 9) {
            return (Document)node;
        }
        try {
            return node.getOwnerDocument();
        }
        catch (NullPointerException ex) {
            throw new NullPointerException(I18n.translate("endorsed.jdk1.4.0") + " Original message was \"" + ex.getMessage() + "\"");
        }
    }
    
    public static Document getOwnerDocument(Set<Node> set) {
        NullPointerException ex = null;
        for (Node node : set) {
            int nodeType = node.getNodeType();
            if (nodeType == 9) {
                return (Document)node;
            }
            try {
                if (nodeType == 2) {
                    return ((Attr)node).getOwnerElement().getOwnerDocument();
                }
                return node.getOwnerDocument();
            }
            catch (NullPointerException ex2) {
                ex = ex2;
            }
        }
        throw new NullPointerException(I18n.translate("endorsed.jdk1.4.0") + " Original message was \"" + ((ex == null) ? "" : ex.getMessage()) + "\"");
    }
    
    public static Element createDSctx(Document document, String s, String s2) {
        if (s == null || s.trim().length() == 0) {
            throw new IllegalArgumentException("You must supply a prefix");
        }
        Element elementNS = document.createElementNS(null, "namespaceContext");
        elementNS.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:" + s.trim(), s2);
        return elementNS;
    }
    
    public static void addReturnToElement(Element element) {
        if (!XMLUtils.ignoreLineBreaks) {
            element.appendChild(element.getOwnerDocument().createTextNode("\n"));
        }
    }
    
    public static void addReturnToElement(Document document, HelperNodeList list) {
        if (!XMLUtils.ignoreLineBreaks) {
            list.appendChild(document.createTextNode("\n"));
        }
    }
    
    public static void addReturnBeforeChild(Element element, Node node) {
        if (!XMLUtils.ignoreLineBreaks) {
            element.insertBefore(element.getOwnerDocument().createTextNode("\n"), node);
        }
    }
    
    public static String encodeToString(byte[] src) {
        if (XMLUtils.ignoreLineBreaks) {
            return Base64.getEncoder().encodeToString(src);
        }
        if (XMLUtils.LINEFEED_ONLY) {
            return Base64.getMimeEncoder(76, new byte[] { 10 }).encodeToString(src);
        }
        return Base64.getMimeEncoder().encodeToString(src);
    }
    
    public static byte[] decode(String src) {
        return Base64.getMimeDecoder().decode(src);
    }
    
    public static byte[] decode(byte[] src) {
        return Base64.getMimeDecoder().decode(src);
    }
    
    public static boolean isIgnoreLineBreaks() {
        return XMLUtils.ignoreLineBreaks;
    }
    
    public static Set<Node> convertNodelistToSet(NodeList list) {
        if (list == null) {
            return new HashSet();
        }
        int length = list.getLength();
        HashSet<Node> set = new HashSet(length);
        for (int i = 0; i < length; ++i) {
            set.add(list.item(i));
        }
        return set;
    }
    
    public static void circumventBug2650(Document document) {
        Element documentElement = document.getDocumentElement();
        if (documentElement.getAttributeNodeNS("http://www.w3.org/2000/xmlns/", "xmlns") == null) {
            documentElement.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns", "");
        }
        circumventBug2650internal(document);
    }
    
    private static void circumventBug2650internal(Node node) {
        Node parentNode = null;
        Node node2 = null;
        while (true) {
            switch (node.getNodeType()) {
                case 1: {
                    Element element = (Element)node;
                    if (!element.hasChildNodes()) {
                        break;
                    }
                    if (element.hasAttributes()) {
                        NamedNodeMap attributes = element.getAttributes();
                        int length = attributes.getLength();
                        for (Node node3 = element.getFirstChild(); node3 != null; node3 = node3.getNextSibling()) {
                            if (node3.getNodeType() == 1) {
                                Element element2 = (Element)node3;
                                for (int i = 0; i < length; ++i) {
                                    Attr attr = (Attr)attributes.item(i);
                                    if ("http://www.w3.org/2000/xmlns/".equals(attr.getNamespaceURI()) && !element2.hasAttributeNS("http://www.w3.org/2000/xmlns/", attr.getLocalName())) {
                                        element2.setAttributeNS("http://www.w3.org/2000/xmlns/", attr.getName(), attr.getNodeValue());
                                    }
                                }
                            }
                        }
                    }
                }
                case 5:
                case 9: {
                    parentNode = node;
                    node2 = node.getFirstChild();
                    break;
                }
            }
            while (node2 == null && parentNode != null) {
                node2 = parentNode.getNextSibling();
                parentNode = parentNode.getParentNode();
            }
            if (node2 == null) {
                break;
            }
            node = node2;
            node2 = node.getNextSibling();
        }
    }
    
    public static Element selectDsNode(Node nextSibling, String anObject, int n) {
        while (nextSibling != null) {
            if ("http://www.w3.org/2000/09/xmldsig#".equals(nextSibling.getNamespaceURI()) && nextSibling.getLocalName().equals(anObject)) {
                if (n == 0) {
                    return (Element)nextSibling;
                }
                --n;
            }
            nextSibling = nextSibling.getNextSibling();
        }
        return null;
    }
    
    public static Element selectDs11Node(Node nextSibling, String anObject, int n) {
        while (nextSibling != null) {
            if ("http://www.w3.org/2009/xmldsig11#".equals(nextSibling.getNamespaceURI()) && nextSibling.getLocalName().equals(anObject)) {
                if (n == 0) {
                    return (Element)nextSibling;
                }
                --n;
            }
            nextSibling = nextSibling.getNextSibling();
        }
        return null;
    }
    
    public static Text selectDsNodeText(Node node, String s, int n) {
        Element selectDsNode = selectDsNode(node, s, n);
        if (selectDsNode == null) {
            return null;
        }
        Node node2;
        for (node2 = selectDsNode.getFirstChild(); node2 != null && node2.getNodeType() != 3; node2 = node2.getNextSibling()) {}
        return (Text)node2;
    }
    
    public static Text selectDs11NodeText(Node node, String s, int n) {
        Element selectDs11Node = selectDs11Node(node, s, n);
        if (selectDs11Node == null) {
            return null;
        }
        Node node2;
        for (node2 = selectDs11Node.getFirstChild(); node2 != null && node2.getNodeType() != 3; node2 = node2.getNextSibling()) {}
        return (Text)node2;
    }
    
    public static Text selectNodeText(Node node, String s, String s2, int n) {
        Element selectNode = selectNode(node, s, s2, n);
        if (selectNode == null) {
            return null;
        }
        Node node2;
        for (node2 = selectNode.getFirstChild(); node2 != null && node2.getNodeType() != 3; node2 = node2.getNextSibling()) {}
        return (Text)node2;
    }
    
    public static Element selectNode(Node nextSibling, String anObject, String anObject2, int n) {
        while (nextSibling != null) {
            if (nextSibling.getNamespaceURI() != null && nextSibling.getNamespaceURI().equals(anObject) && nextSibling.getLocalName().equals(anObject2)) {
                if (n == 0) {
                    return (Element)nextSibling;
                }
                --n;
            }
            nextSibling = nextSibling.getNextSibling();
        }
        return null;
    }
    
    public static Element[] selectDsNodes(Node node, String s) {
        return selectNodes(node, "http://www.w3.org/2000/09/xmldsig#", s);
    }
    
    public static Element[] selectDs11Nodes(Node node, String s) {
        return selectNodes(node, "http://www.w3.org/2009/xmldsig11#", s);
    }
    
    public static Element[] selectNodes(Node nextSibling, String anObject, String anObject2) {
        ArrayList<Node> list = new ArrayList();
        while (nextSibling != null) {
            if (nextSibling.getNamespaceURI() != null && nextSibling.getNamespaceURI().equals(anObject) && nextSibling.getLocalName().equals(anObject2)) {
                list.add((Node)nextSibling);
            }
            nextSibling = nextSibling.getNextSibling();
        }
        return (Element[])list.toArray(new Element[list.size()]);
    }
    
    public static Set<Node> excludeNodeFromSet(Node node, Set<Node> set) {
        HashSet<Node> set2 = new HashSet();
        for (Node node2 : set) {
            if (!isDescendantOrSelf(node, node2)) {
                set2.add(node2);
            }
            else {
                continue;
            }
        }
        return set2;
    }
    
    public static String getStrFromNode(Node node) {
        if (node.getNodeType() == 3) {
            StringBuilder sb = new StringBuilder();
            for (Node node2 = node.getParentNode().getFirstChild(); node2 != null; node2 = node2.getNextSibling()) {
                if (node2.getNodeType() == 3) {
                    sb.append(((Text)node2).getData());
                }
            }
            return sb.toString();
        }
        if (node.getNodeType() == 2) {
            return node.getNodeValue();
        }
        if (node.getNodeType() == 7) {
            return node.getNodeValue();
        }
        return null;
    }
    
    public static boolean isDescendantOrSelf(Node node, Node node2) {
        if (node == node2) {
            return true;
        }
        while (node2 != null) {
            if (node2 == node) {
                return true;
            }
            if (node2.getNodeType() == 2) {
                node2 = ((Attr)node2).getOwnerElement();
            }
            else {
                node2 = node2.getParentNode();
            }
        }
        return false;
    }
    
    public static boolean ignoreLineBreaks() {
        return XMLUtils.ignoreLineBreaks;
    }
    
    public static String getAttributeValue(Element element, String s) {
        Attr attributeNodeNS = element.getAttributeNodeNS(null, s);
        return (attributeNodeNS == null) ? null : attributeNodeNS.getValue();
    }
    
    public static boolean protectAgainstWrappingAttack(Node node, String s) {
        String s2 = s.trim();
        if (!s2.isEmpty() && s2.charAt(0) == '#') {
            s2 = s2.substring(1);
        }
        Node parentNode = null;
        Element ownerElement = null;
        if (node != null) {
            parentNode = node.getParentNode();
        }
        while (node != null) {
            if (node.getNodeType() == 1) {
                NamedNodeMap attributes = ((Node)node).getAttributes();
                if (attributes != null) {
                    for (int length = attributes.getLength(), i = 0; i < length; ++i) {
                        Attr attr = (Attr)attributes.item(i);
                        if (attr.isId() && s2.equals(attr.getValue())) {
                            if (ownerElement == null) {
                                ownerElement = attr.getOwnerElement();
                            }
                            else {
                                XMLUtils.LOG.debug("Multiple elements with the same 'Id' attribute value!");
                                return false;
                            }
                        }
                    }
                }
            }
            Node parentNode2 = node;
            node = node.getFirstChild();
            if (node == null) {
                node = parentNode2.getNextSibling();
            }
            while (node == null) {
                parentNode2 = parentNode2.getParentNode();
                if (parentNode2 == parentNode) {
                    return true;
                }
                node = parentNode2.getNextSibling();
            }
        }
        return true;
    }
    
    public static boolean protectAgainstWrappingAttack(Node node, Element element, String s) {
        String s2 = s.trim();
        if (!s2.isEmpty() && s2.charAt(0) == '#') {
            s2 = s2.substring(1);
        }
        Node parentNode = null;
        if (node != null) {
            parentNode = node.getParentNode();
        }
        while (node != null) {
            if (node.getNodeType() == 1) {
                Element element2 = (Element)node;
                NamedNodeMap attributes = element2.getAttributes();
                if (attributes != null) {
                    int length = attributes.getLength();
                    int i = 0;
                    while (i < length) {
                        Attr attr = (Attr)attributes.item(i);
                        if (attr.isId() && s2.equals(attr.getValue()) && element2 != element) {
                            XMLUtils.LOG.debug("Multiple elements with the same 'Id' attribute value!");
                            return false;
                        }
                        ++i;
                    }
                }
            }
            Node parentNode2 = node;
            node = node.getFirstChild();
            if (node == null) {
                node = parentNode2.getNextSibling();
            }
            while (node == null) {
                parentNode2 = parentNode2.getParentNode();
                if (parentNode2 == parentNode) {
                    return true;
                }
                node = parentNode2.getNextSibling();
            }
        }
        return true;
    }
    
    public static DocumentBuilder createDocumentBuilder(boolean b) throws ParserConfigurationException {
        return createDocumentBuilder(b, true);
    }
    
    public static DocumentBuilder createDocumentBuilder(boolean validating, boolean b) throws ParserConfigurationException {
        DocumentBuilderFactory instance = DocumentBuilderFactory.newInstance();
        instance.setFeature("http://javax.xml.XMLConstants/feature/secure-processing", true);
        if (b) {
            instance.setFeature("http://apache.org/xml/features/disallow-doctype-decl", true);
        }
        instance.setValidating(validating);
        instance.setNamespaceAware(true);
        return instance.newDocumentBuilder();
    }
    
    public static byte[] getBytes(BigInteger bigInteger, int n) {
        n = n + 7 >> 3 << 3;
        if (n < bigInteger.bitLength()) {
            throw new IllegalArgumentException(I18n.translate("utils.Base64.IllegalBitlength"));
        }
        byte[] byteArray = bigInteger.toByteArray();
        if (bigInteger.bitLength() % 8 != 0 && bigInteger.bitLength() / 8 + 1 == n / 8) {
            return byteArray;
        }
        int n2 = 0;
        int length = byteArray.length;
        if (bigInteger.bitLength() % 8 == 0) {
            n2 = 1;
            --length;
        }
        int n3 = n / 8 - length;
        byte[] array = new byte[n / 8];
        System.arraycopy(byteArray, n2, array, n3, length);
        return array;
    }
    
    static {
        XMLUtils.ignoreLineBreaks = (boolean)AccessController.doPrivileged(()/*  */ -> Boolean.valueOf(Boolean.getBoolean("com.sun.org.apache.xml.internal.security.ignoreLineBreaks")));
        LINEFEED_ONLY = (boolean)AccessController.doPrivileged(()/*  */ -> Boolean.valueOf(Boolean.getBoolean("com.sun.org.apache.xml.internal.security.lineFeedOnly")));
        XMLUtils.dsPrefix = "ds";
        XMLUtils.ds11Prefix = "dsig11";
        XMLUtils.xencPrefix = "xenc";
        XMLUtils.xenc11Prefix = "xenc11";
        LOG = LoggerFactory.getLogger(XMLUtils.class);
    }
}
